"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignalAgent = void 0;
const sysUtil_1 = require("../../util/sysUtil");
const signalProxy_1 = require("./signalProxy");
const bayLog_1 = require("../../bayLog");
const bayMessage_1 = require("../../bayMessage");
const symbol_1 = require("../../symbol");
const strUtil_1 = require("../../util/strUtil");
const grandAgentMonitor_1 = require("../grandAgentMonitor");
const net_1 = require("net");
class SignalAgent {
    constructor(port) {
        this.port = port;
        bayLog_1.BayLog.info(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_OPEN_CTL_PORT, port));
        this.serverSocket = new net_1.Server((skt) => {
            skt.on("data", (buf) => {
                let cmd = buf.toString().trim();
                SignalAgent.handleCommand(cmd);
                skt.write("OK");
            });
        });
        this.serverSocket.listen(port, () => {
            bayLog_1.BayLog.info("Port opened: %s", this.serverSocket.address());
        });
        this.serverSocket.on("error", (err) => {
            bayLog_1.BayLog.error("Open failed");
            bayLog_1.BayLog.error_e(err);
        });
    }
    static init(port) {
        if (port > 0) {
            this.signalAgent = new SignalAgent(port);
        }
        else {
            let sp = signalProxy_1.SignalProxy.getProxy();
            if (sp != null) {
                for (const cmd of this.commands) {
                    sp.register(this.getSignalFromCommand(cmd), () => {
                        this.handleCommand(cmd);
                    });
                }
            }
        }
    }
    static initSignalMap() {
        if (this.signalMap != null)
            return;
        this.signalMap = new Map();
        if (sysUtil_1.SysUtil.runOnWindows()) {
            /** Available signals on Windows
             SIGABRT
             SIGFPE
             SIGILL
             SIGINT
             SIGSEGV
             SIGTERM
             */
            this.signalMap.set("SIGSEGV", this.COMMAND_RELOAD_CERT);
            this.signalMap.set("SIGILL", this.COMMAND_MEM_USAGE);
            this.signalMap.set("SIGINT", this.COMMAND_SHUTDOWN);
            this.signalMap.set("SIGTERM", this.COMMAND_RESTART_AGENTS);
            this.signalMap.set("SIGABRT", this.COMMAND_ABORT);
        }
        else {
            this.signalMap.set("SIGALRM", this.COMMAND_RELOAD_CERT);
            this.signalMap.set("SIGTRAP", this.COMMAND_MEM_USAGE);
            this.signalMap.set("SIGHUP", this.COMMAND_RESTART_AGENTS);
            this.signalMap.set("SIGTERM", this.COMMAND_SHUTDOWN);
            this.signalMap.set("SIGABRT", this.COMMAND_ABORT);
        }
    }
    static handleCommand(cmd) {
        bayLog_1.BayLog.debug("handle command: %s", cmd);
        try {
            switch (cmd.toLowerCase()) {
                case this.COMMAND_RELOAD_CERT:
                    grandAgentMonitor_1.GrandAgentMonitor.reloadCertAll();
                    break;
                case this.COMMAND_MEM_USAGE:
                    grandAgentMonitor_1.GrandAgentMonitor.printUsageAll();
                    break;
                case this.COMMAND_RESTART_AGENTS:
                    try {
                        grandAgentMonitor_1.GrandAgentMonitor.restartAll();
                    }
                    catch (e) {
                        bayLog_1.BayLog.error(e);
                    }
                    break;
                case this.COMMAND_SHUTDOWN:
                    grandAgentMonitor_1.GrandAgentMonitor.shutdownAll();
                    break;
                case this.COMMAND_ABORT:
                    grandAgentMonitor_1.GrandAgentMonitor.abortAll();
                    break;
                default:
                    bayLog_1.BayLog.error("Unknown command: " + cmd);
                    break;
            }
        }
        catch (e) {
            bayLog_1.BayLog.error_e(e);
        }
    }
    static getSignalFromCommand(command) {
        this.initSignalMap();
        for (const [sig, com] of this.signalMap) {
            if (strUtil_1.StrUtil.eqIgnoreCase(com, command))
                return sig;
        }
        return null;
    }
}
exports.SignalAgent = SignalAgent;
_a = SignalAgent;
SignalAgent.COMMAND_RELOAD_CERT = "reloadcert";
SignalAgent.COMMAND_MEM_USAGE = "memusage";
SignalAgent.COMMAND_RESTART_AGENTS = "restartagents";
SignalAgent.COMMAND_SHUTDOWN = "shutdown";
SignalAgent.COMMAND_ABORT = "abort";
SignalAgent.commands = [
    _a.COMMAND_RELOAD_CERT,
    _a.COMMAND_MEM_USAGE,
    _a.COMMAND_RESTART_AGENTS,
    _a.COMMAND_SHUTDOWN,
    _a.COMMAND_ABORT
];
SignalAgent.signalMap = null;
//# sourceMappingURL=signalAgent.js.map