"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignalSender = void 0;
const bayserver_1 = require("../../bayserver");
const signalAgent_1 = require("./signalAgent");
const bayException_1 = require("../../bayException");
const bayLog_1 = require("../../bayLog");
const bayMessage_1 = require("../../bayMessage");
const symbol_1 = require("../../symbol");
const bcfParser_1 = require("../../bcf/bcfParser");
const bcfElement_1 = require("../../bcf/bcfElement");
const strUtil_1 = require("../../util/strUtil");
const bcfKeyVal_1 = require("../../bcf/bcfKeyVal");
const net = require("net");
const fs = require("fs");
const process_1 = require("process");
class SignalSender {
    constructor() {
        this.bayPort = -1;
        this.pidFile = "bayserver.pid";
    }
    /**
     * Send running BayServer a command
     */
    sendCommand(cmd) {
        this.parseBayPort(bayserver_1.BayServer.bservPlan);
        if (this.bayPort < 0) {
            let pid = this.readPidFile();
            let sig = signalAgent_1.SignalAgent.getSignalFromCommand(cmd);
            if (sig == null)
                throw new bayException_1.BayException("Invalid command: " + cmd);
            if (pid <= 0)
                throw new bayException_1.BayException("Invalid process ID: " + pid);
            this.kill(pid, sig);
        }
        else {
            bayLog_1.BayLog.info(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_SENDING_COMMAND, cmd));
            this.send("localhost", this.bayPort, cmd);
        }
    }
    /**
     * Parse plan file and get port number of SignalAgent
     */
    parseBayPort(plan) {
        let p = new bcfParser_1.BcfParser();
        let doc = p.parse(plan);
        for (const o of doc.contentList) {
            if (o instanceof bcfElement_1.BcfElement) {
                let elm = o;
                if (strUtil_1.StrUtil.eqIgnoreCase(elm.name, "harbor")) {
                    for (const o2 of elm.contentList) {
                        if (o2 instanceof bcfKeyVal_1.BcfKeyVal) {
                            let kv = o2;
                            if (strUtil_1.StrUtil.eqIgnoreCase(kv.key, "controlPort"))
                                this.bayPort = Number.parseInt(kv.value);
                            else if (strUtil_1.StrUtil.eqIgnoreCase(kv.key, "pidFile"))
                                this.pidFile = kv.value;
                        }
                    }
                }
            }
        }
    }
    /**
     * Send another BayServer running host:port a command
     */
    send(host, port, cmd) {
        let skt = new net.Socket();
        skt.setTimeout(30 * 1000);
        skt.connect(port, host, () => {
            skt.write(cmd + "\n", (err) => {
            });
            skt.on("data", (buf) => {
                skt.end(() => {
                });
                (0, process_1.exit)(0);
            });
        });
        skt.on("error", (err) => {
            bayLog_1.BayLog.error("Connect error");
            bayLog_1.BayLog.error_e(err);
            (0, process_1.exit)(0);
        });
    }
    readPidFile() {
        let buf = fs.readFileSync(bayserver_1.BayServer.getLocation(this.pidFile));
        return Number.parseInt(buf.toString());
    }
    kill(pid, sig) {
        try {
            process.kill(pid, sig);
        }
        catch (e) {
            bayLog_1.BayLog.error_e(e);
        }
    }
}
exports.SignalSender = SignalSender;
//# sourceMappingURL=signalSender.js.map