"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BayServer = void 0;
const process_1 = require("process");
const bayException_1 = require("./bayException");
const bayLog_1 = require("./bayLog");
const sysUtil_1 = require("./util/sysUtil");
const path_1 = require("path");
require("source-map-support/register");
const bayMessage_1 = require("./bayMessage");
const locale_1 = require("./util/locale");
const bayDockers_1 = require("./bayDockers");
const mimes_1 = require("./util/mimes");
const httpStatus_1 = require("./util/httpStatus");
const bcfParser_1 = require("./bcf/bcfParser");
const bcfElement_1 = require("./bcf/bcfElement");
const signalAgent_1 = require("./agent/signal/signalAgent");
const version_1 = require("./version");
const os_1 = require("os");
const symbol_1 = require("./symbol");
const portMapItem_1 = require("./agent/portMapItem");
const grandAgent_1 = require("./agent/grandAgent");
const strUtil_1 = require("./util/strUtil");
const net = require("net");
const path = require("path");
const cities_1 = require("./util/cities");
const packetStore_1 = require("./protocol/packetStore");
const inboundShipStore_1 = require("./docker/base/inboundShipStore");
const protocolHandlerStore_1 = require("./protocol/protocolHandlerStore");
const tourStore_1 = require("./tour/tourStore");
const memUsage_1 = require("./memUsage");
const grandAgentMonitor_1 = require("./agent/grandAgentMonitor");
const fs = require("fs");
const signalSender_1 = require("./agent/signal/signalSender");
const md5Password_1 = require("./util/md5Password");
const ioException_1 = require("./util/ioException");
class BayServer {
    static main(args) {
        //BayLog.info("main: %s", args.join(","))
        this.scriptName = args[1];
        this.scriptArgs = [...args];
        this.scriptArgs.splice(0, 2);
        let cmd = null;
        let home = process_1.env.BSERV_HOME;
        let plan = process_1.env.BSERV_PLAN;
        let mkpass = null;
        let agentId = -1;
        let openPort = false;
        let monitorPort = -1;
        let init = false;
        for (const arg of args) {
            if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-start"))
                cmd = null;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-stop") || strUtil_1.StrUtil.eqIgnoreCase(arg, "-shutdown"))
                cmd = signalAgent_1.SignalAgent.COMMAND_SHUTDOWN;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-restartAgents"))
                cmd = signalAgent_1.SignalAgent.COMMAND_RESTART_AGENTS;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-reloadCert"))
                cmd = signalAgent_1.SignalAgent.COMMAND_RELOAD_CERT;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-memUsage"))
                cmd = signalAgent_1.SignalAgent.COMMAND_MEM_USAGE;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-abort"))
                cmd = signalAgent_1.SignalAgent.COMMAND_ABORT;
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-init"))
                init = true;
            else if (arg.toLowerCase().startsWith("-home="))
                home = arg.substring(6);
            else if (arg.toLowerCase().startsWith("-plan="))
                plan = arg.substring(6);
            else if (arg.toLowerCase().startsWith("-mkpass="))
                mkpass = arg.substring(8);
            else if (arg.toLowerCase().startsWith("-loglevel="))
                bayLog_1.BayLog.setLogLevel(arg.substring(10));
            else if (arg.toLowerCase().startsWith("-agentid="))
                agentId = Number.parseInt(arg.substring(9));
            else if (strUtil_1.StrUtil.eqIgnoreCase(arg, "-openPort"))
                openPort = true;
            else if (arg.toLowerCase().startsWith("-monitorport="))
                monitorPort = Number.parseInt(arg.substring(13));
        }
        if (mkpass != null) {
            console.log(md5Password_1.Md5Password.encode(mkpass));
            return;
        }
        Error.stackTraceLimit = Infinity;
        if (agentId > 0)
            bayLog_1.BayLog.debug("agt#%d Child process started", agentId);
        this.getHome(home);
        this.getLib();
        if (init) {
            this.init();
        }
        else {
            this.getPlan(plan);
            if (cmd == null) {
                BayServer.start(agentId, openPort, monitorPort);
            }
            else {
                new signalSender_1.SignalSender().sendCommand(cmd);
            }
        }
    }
    static getHome(home) {
        // Get BayServer home
        if (home == null)
            home = ".";
        this.bservHome = sysUtil_1.SysUtil.getAbsolutePath(home);
        if (!sysUtil_1.SysUtil.isDirectory(this.bservHome))
            throw new bayException_1.BayException("BayServer home is not a directory: %s", this.bservHome);
        if (this.bservHome.endsWith(path_1.sep))
            this.bservHome = this.bservHome.substring(0, this.bservHome.length - 1);
        bayLog_1.BayLog.debug("BayServer home: %s", this.bservHome);
    }
    static getPlan(plan) {
        // Get debug mode
        //BayLog.debug("Log level=" + BayLog.logLevel);
        // Get plan file
        if (plan == null)
            plan = "plan/bayserver.plan";
        if (!sysUtil_1.SysUtil.isAbsolutePath(plan))
            plan = this.bservHome + path_1.sep + plan;
        this.bservPlan = sysUtil_1.SysUtil.getAbsolutePath(plan);
        bayLog_1.BayLog.info("BayServer Plan: %s", this.bservPlan);
        if (!sysUtil_1.SysUtil.isFile(this.bservPlan))
            throw new bayException_1.BayException("Plan file is not a file: %s", this.bservPlan);
    }
    static getLib() {
        this.bservLib = path.join(this.bservHome, "node_modules", "bayserver");
        if (!sysUtil_1.SysUtil.isDirectory(this.bservLib))
            throw new bayException_1.BayException("Library directory is not a directory: %s", this.bservLib);
        bayLog_1.BayLog.debug("BayServer lib: %s", this.bservLib);
    }
    static init() {
        let initDir = path.join(this.bservLib, "init");
        bayLog_1.BayLog.debug("init directory: %s", initDir);
        sysUtil_1.SysUtil.copyDir(initDir, this.bservHome);
    }
    static start(agentId, openPort, monitorPort) {
        try {
            bayMessage_1.BayMessage.init(this.bservLib + "/conf/messages", new locale_1.Locale("ja", "JP"));
            bayDockers_1.BayDockers.init(this.bservLib + "/conf/dockers.bcf");
            mimes_1.Mimes.init(this.bservLib + "/conf/mimes.bcf");
            httpStatus_1.HttpStatus.init(this.bservLib + "/conf/httpstatus.bcf");
            this.loadPlan(this.bservPlan);
            /** Init stores, memory usage managers */
            packetStore_1.PacketStore.init();
            inboundShipStore_1.InboundShipStore.init();
            protocolHandlerStore_1.ProtocolHandlerStore.init();
            tourStore_1.TourStore.init(tourStore_1.TourStore.MAX_TOURS);
            memUsage_1.MemUsage.init();
            let redirectFile = this.harbor.getRedirectFile();
            if (strUtil_1.StrUtil.isSet(redirectFile)) {
                redirectFile = this.getLocation(redirectFile);
                if (BayServer.harbor.isMultiCore() && agentId > 0) {
                    let p = redirectFile.lastIndexOf('.');
                    let filePrefix, fileExt;
                    if (p == -1) {
                        filePrefix = redirectFile;
                        fileExt = "";
                    }
                    else {
                        filePrefix = redirectFile.substring(0, p);
                        fileExt = redirectFile.substring(p);
                    }
                    redirectFile = filePrefix + "-" + agentId + fileExt;
                }
                let newConsole = fs.createWriteStream(redirectFile);
                newConsole.on('error', (err) => {
                    bayLog_1.BayLog.error_e(err, bayMessage_1.BayMessage.get(symbol_1.Symbol.CFG_CANNOT_OPEN_REDIRECT_FILE, redirectFile));
                    throw new Error(bayMessage_1.BayMessage.get(symbol_1.Symbol.CFG_CANNOT_OPEN_REDIRECT_FILE, redirectFile));
                });
                newConsole.on('open', () => {
                    bayLog_1.BayLog.debug(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_REDIRECT_FILE_OPENED, redirectFile));
                    console.log = (...args) => {
                        let message = args.join(' ');
                        newConsole.write(message + '\n');
                    };
                    console.error = console.log;
                });
            }
            if (agentId == -1) {
                this.printVersion();
                this.myHostName = (0, os_1.hostname)();
                bayLog_1.BayLog.debug("Host name    : " + this.myHostName);
                this.parentStart();
            }
            else {
                this.childStart(agentId, openPort, monitorPort);
            }
        }
        catch (e) {
            bayLog_1.BayLog.error_e(e);
            (0, process_1.exit)(1);
        }
    }
    static openPorts(anchoredPortMap, callback) {
        let curIndex = 0;
        for (const portDkr of this.ports) {
            // Open ports
            if (portDkr.isAnchored()) {
                // Open TCP port
                bayLog_1.BayLog.info(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_OPENING_TCP_PORT, portDkr.getHost() == null ? "" : portDkr.getHost(), portDkr.getPort(), portDkr.protocol()));
                let ch = portDkr.createServer();
                ch.on('error', (err) => {
                    bayLog_1.BayLog.error("Server socket error: %s", err);
                });
                /*
                ch.on('connection', (socket: net.Socket) => {
                    let agt: GrandAgent = null
                    try {
                        agt = GrandAgent.getByIndex(curIndex)
                        BayLog.debug("%s Accepted on parent process", agt)
                        agt.acceptHandler.onAccept(ch, socket)
                        if(!BayServer.harbor.isMultiCore())
                            curIndex = (curIndex + 1) % BayServer.harbor.getNumGrandAgents()
                    }
                    catch(e) {
                        BayLog.error_e(e)
                        if(agt) {
                            agt.abort(e)
                        }
                    }
                });
                */
                ch.listen(portDkr.getPort(), () => {
                    anchoredPortMap.push(new portMapItem_1.PortMapItem(ch, portDkr));
                    if (anchoredPortMap.length == this.ports.length) {
                        callback();
                    }
                });
            }
            else {
                throw new bayException_1.BayException();
            }
        }
    }
    static parentStart() {
        let anchoredPortMap = []; // TCP server port map
        if (!BayServer.harbor.isMultiCore()) {
            // Single core not supported!
            // Single core mode
            this.openPorts(anchoredPortMap, () => {
            });
            // Single core mode
            grandAgent_1.GrandAgent.init(Array.from({ length: BayServer.harbor.getNumGrandAgents() }, (_, index) => index + 1), anchoredPortMap, null, BayServer.harbor.getMaxShips(), BayServer.harbor.isMultiCore());
        }
        grandAgentMonitor_1.GrandAgentMonitor.init(this.harbor.getNumGrandAgents(), anchoredPortMap);
        signalAgent_1.SignalAgent.init(this.harbor.getControlPort());
        this.createPidFile(sysUtil_1.SysUtil.pid());
    }
    static childStart(agentId, openPort, monitorPort) {
        let anchoredPortMap = []; // TCP server port map
        //BayLog.info("child start: %d open=%s monitor=%d", agentId, openPort, monitorPort)
        if (openPort) {
            //BayLog.info("agt#%d open port mode", agentId)
            let anchoredPortMap = []; // TCP server port map
            this.openPorts(anchoredPortMap, () => {
                this.initChildGrandAgent(agentId, anchoredPortMap);
                this.connectToMonitor(agentId, monitorPort);
            });
        }
        else {
            //BayLog.info("agt#%d recv socket mode", agentId)
            process.on('message', (m, skt) => {
                if (m.startsWith('serverSocket')) {
                    let adr = skt.address();
                    let host = adr["address"];
                    let port = adr["port"];
                    var portDkr = null;
                    //BayLog.info('agt#%d Received server Socket', agentId);
                    for (const p of this.ports) {
                        if (p.getPort() == port) {
                            portDkr = p;
                            break;
                        }
                    }
                    if (portDkr == null) {
                        bayLog_1.BayLog.fatal("Cannot find port docker: " + port);
                        (0, process_1.exit)(1);
                    }
                    else {
                        anchoredPortMap.push(new portMapItem_1.PortMapItem(skt, portDkr));
                        if (anchoredPortMap.length == this.ports.length) {
                            // All ports are opened
                            this.initChildGrandAgent(agentId, anchoredPortMap);
                            this.connectToMonitor(agentId, monitorPort);
                        }
                    }
                }
            });
        }
    }
    static initChildGrandAgent(agentId, anchoredPortMap) {
        grandAgent_1.GrandAgent.init([agentId], anchoredPortMap, null, this.harbor.getMaxShips(), this.harbor.isMultiCore());
        let agt = grandAgent_1.GrandAgent.get(agentId);
        for (const portMap of grandAgent_1.GrandAgent.anchorablePortMap) {
            portMap.ch.on('connection', (socket) => {
                bayLog_1.BayLog.info("Accepted");
                agt.acceptHandler.onAccept(portMap.ch, socket);
            });
        }
    }
    static connectToMonitor(agentId, monitorPort) {
        //BayLog.debug("Connect to monitor: port=%d", monitorPort)
        const skt = net.createConnection({ host: "localhost", port: monitorPort }, () => {
            //BayLog.debug("Connected to monitor: port=%d", monitorPort)
            let agt = grandAgent_1.GrandAgent.get(agentId);
            agt.runCommandReceiver(skt);
        });
        skt.on('error', err => {
            bayLog_1.BayLog.error_e(err);
        });
    }
    static findCity(name) {
        return this.cities.findCity(name);
    }
    static parsePath(location) {
        location = this.getLocation(location);
        if (!sysUtil_1.SysUtil.exists(location))
            throw new ioException_1.IOException("File not found: " + location);
        return location;
    }
    static getLocation(location) {
        if (!sysUtil_1.SysUtil.isAbsolutePath(location))
            return this.bservHome + path.sep + location;
        else
            return location;
    }
    //////////////////////////////////////////////////////
    // private methods
    //////////////////////////////////////////////////////
    static loadPlan(bservConf) {
        let p = new bcfParser_1.BcfParser();
        let doc = p.parse(bservConf);
        for (const o of doc.contentList) {
            if (o instanceof bcfElement_1.BcfElement) {
                let dkr = bayDockers_1.BayDockers.createDocker(o, null);
                if (dkr.getType() == "port") {
                    BayServer.ports.push(dkr);
                }
                else if (dkr.getType() == "harbor") {
                    BayServer.harbor = dkr;
                }
                else if (dkr.getType() == "city") {
                    BayServer.cities.add(dkr);
                }
            }
        }
    }
    /**
     * Print version information
     */
    static printVersion() {
        let version = "Version " + this.getVersion();
        while (version.length < 28)
            version = ' ' + version;
        console.log("        ----------------------");
        console.log("       /     BayServer        \\");
        console.log("-----------------------------------------------------");
        var line = " \\";
        var i;
        for (i = 0; i < 47 - version.length; i++)
            line += " ";
        console.log(line + version + "  /");
        console.log("  \\           Copyright (C) 2000 Yokohama Baykit  /");
        console.log("   \\                     http://baykit.yokohama  /");
        console.log("    ---------------------------------------------");
    }
    static getVersion() {
        return version_1.Version.VERSION;
    }
    static getSoftwareName() {
        if (this.softwareName == null)
            this.softwareName = "BayServer/" + this.getVersion();
        return this.softwareName;
    }
    static createPidFile(pid) {
        fs.writeFileSync(this.getLocation(this.harbor.getPidFile()), pid.toString());
    }
}
exports.BayServer = BayServer;
BayServer.cities = new cities_1.Cities();
/** Port dockers */
BayServer.ports = [];
//# sourceMappingURL=bayserver.js.map