"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClubBase = void 0;
const dockerBase_1 = require("./dockerBase");
const strUtil_1 = require("../../util/strUtil");
class ClubBase extends dockerBase_1.DockerBase {
    constructor() {
        super(...arguments);
        this.decodePathInfo = true;
    }
    toString() {
        return "clb$" + this.constructor.name;
    }
    //////////////////////////////////////////////////////
    // Implements Docker
    //////////////////////////////////////////////////////
    init(elm, parent) {
        super.init(elm, parent);
        let p = elm.arg.lastIndexOf('.');
        if (p == -1) {
            this.fileName = elm.arg;
            this.extension = null;
        }
        else {
            this.fileName = elm.arg.substring(0, p);
            this.extension = elm.arg.substring(p + 1);
        }
    }
    //////////////////////////////////////////////////////
    // Implements DockerBase
    //////////////////////////////////////////////////////
    initKeyVal(kv) {
        switch (kv.key.toLowerCase()) {
            default:
                return super.initKeyVal(kv);
            case "decodepathinfo":
                this.decodePathInfo = strUtil_1.StrUtil.parseBool(kv.value);
                break;
            case "charset":
                let cs = kv.value;
                if (strUtil_1.StrUtil.isSet(cs))
                    this.charset = cs;
                break;
        }
        return true;
    }
    //////////////////////////////////////////////////////
    // Implements Club
    //////////////////////////////////////////////////////
    getFileName() {
        return this.fileName;
    }
    getExtension() {
        return this.extension;
    }
    getCharset() {
        return this.charset;
    }
    getrDecodePathInfo() {
        return this.decodePathInfo;
    }
    matches(fname) {
        // check club
        let pos = fname.indexOf(".");
        if (pos == -1) {
            // fname has no extension
            if (this.extension != null)
                return false;
            if (this.fileName == "*")
                return true;
            return fname == this.fileName;
        }
        else {
            //fname has extension
            if (this.extension == null)
                return false;
            let nm = fname.substring(0, pos);
            let ext = fname.substring(pos + 1);
            if (this.extension != "*" && ext != this.extension)
                return false;
            if (this.fileName == "*")
                return true;
            else
                return nm == this.fileName;
        }
    }
}
exports.ClubBase = ClubBase;
//# sourceMappingURL=clubBase.js.map