"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DockerBase = void 0;
const bcfKeyVal_1 = require("../../bcf/bcfKeyVal");
const configException_1 = require("../../configException");
const bayMessage_1 = require("../../bayMessage");
const symbol_1 = require("../../symbol");
const bayDockers_1 = require("../../bayDockers");
const bayException_1 = require("../../bayException");
const bayLog_1 = require("../../bayLog");
class DockerBase {
    //////////////////////////////////////////////////////
    // Implements Docker
    //////////////////////////////////////////////////////
    init(elm, parent) {
        this.type = elm.name;
        for (const o of elm.contentList) {
            if (o instanceof bcfKeyVal_1.BcfKeyVal) {
                let kv = o;
                try {
                    if (!this.initKeyVal(kv))
                        throw new configException_1.ConfigException(o.fileName, o.lineNo, bayMessage_1.BayMessage.get(symbol_1.Symbol.CFG_INVALID_PARAMETER, o.key));
                }
                catch (e) {
                    if (e instanceof configException_1.ConfigException)
                        throw e;
                    else {
                        bayLog_1.BayLog.error_e(e);
                        throw new configException_1.ConfigException(kv.fileName, kv.lineNo, e);
                    }
                }
            }
            else {
                let element = o;
                var dkr;
                try {
                    dkr = bayDockers_1.BayDockers.createDocker(element, this);
                }
                catch (e) {
                    let b = e instanceof bayException_1.BayException;
                    if (e instanceof configException_1.ConfigException)
                        throw e;
                    else {
                        bayLog_1.BayLog.error_e(e);
                        throw new configException_1.ConfigException(element.fileName, element.lineNo, bayMessage_1.BayMessage.get(symbol_1.Symbol.CFG_INVALID_DOCKER, element.name));
                    }
                }
                if (!this.initDocker(dkr))
                    throw new configException_1.ConfigException(o.fileName, o.lineNo, bayMessage_1.BayMessage.get(symbol_1.Symbol.CFG_INVALID_DOCKER, o.name));
            }
        }
    }
    getType() {
        return this.type;
    }
    //////////////////////////////////////////////////////
    // Base methods
    //////////////////////////////////////////////////////
    initDocker(dkr) {
        return false;
    }
    initKeyVal(kv) {
        switch (kv.key.toLowerCase()) {
            default:
                return false;
            case "docker":
                return true;
        }
    }
}
exports.DockerBase = DockerBase;
//# sourceMappingURL=dockerBase.js.map