"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InboundDataListener = void 0;
const bayserver_1 = require("../../bayserver");
const bayLog_1 = require("../../bayLog");
const nextSocketAction_1 = require("../../agent/nextSocketAction");
class InboundDataListener {
    constructor(ship) {
        this.ship = ship;
    }
    toString() {
        return this.ship.toString();
    }
    //////////////////////////////////////////////////////
    // Implements DataListener
    //////////////////////////////////////////////////////
    checkTimeout(durationSec) {
        var timeout;
        if (this.ship.socketTimeoutSec <= 0)
            timeout = false;
        else if (this.ship.keeping)
            timeout = durationSec >= bayserver_1.BayServer.harbor.getKeepTimeoutSec();
        else
            timeout = durationSec >= this.ship.socketTimeoutSec;
        bayLog_1.BayLog.debug("%s Check timeout: dur=%d, timeout=%s, keeping=%s limit=%d keeplim=%d", this, durationSec, timeout, this.ship.keeping, this.ship.socketTimeoutSec, bayserver_1.BayServer.harbor.getKeepTimeoutSec());
        return timeout;
    }
    notifyConnect() {
        throw new Error();
    }
    notifyRead(buf, adr) {
        return this.ship.protocolHandler.bytesReceived(buf);
    }
    notifyError(err) {
        bayLog_1.BayLog.debug_e(err, "%s error detected: %s", this, err.message);
    }
    notifyEof() {
        bayLog_1.BayLog.debug("%s EOF detected", this);
        return nextSocketAction_1.NextSocketAction.CLOSE;
    }
    notifyHandshakeDone(protocol) {
        return 0;
    }
    notifyProtocolError(e) {
        bayLog_1.BayLog.debug_e(e, "%s Protocol error: %s", this, e.message);
        return this.ship.protocolHandler.sendReqProtocolError(e);
    }
    notifyClose() {
        bayLog_1.BayLog.debug("%s notifyClose", this);
        this.ship.abortTours();
        if (this.ship.activeTours.length > 0) {
            // cannot close because there are some running tours
            bayLog_1.BayLog.debug(this + " cannot end ship because there are some running tours (ignore)");
            this.ship.needEnd = true;
        }
        else {
            this.ship.endShip();
        }
    }
}
exports.InboundDataListener = InboundDataListener;
//# sourceMappingURL=inboundDataListener.js.map