"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuiltInCityDocker = exports.MatchInfo = exports.ClubMatchInfo = void 0;
const dockerBase_1 = require("../base/dockerBase");
const town_1 = require("../town");
const bayLog_1 = require("../../bayLog");
const bayMessage_1 = require("../../bayMessage");
const symbol_1 = require("../../symbol");
const httpException_1 = require("../../httpException");
const httpStatus_1 = require("../../util/httpStatus");
const bayserver_1 = require("../../bayserver");
const strUtil_1 = require("../../util/strUtil");
const sysUtil_1 = require("../../util/sysUtil");
const bayDockers_1 = require("../../bayDockers");
class ClubMatchInfo {
}
exports.ClubMatchInfo = ClubMatchInfo;
class MatchInfo {
}
exports.MatchInfo = MatchInfo;
class BuiltInCityDocker extends dockerBase_1.DockerBase {
    constructor() {
        super(...arguments);
        this.townList = [];
        this.clubList = [];
        this.logList = [];
        this.permissionList = [];
    }
    toString() {
        return "cty$" + this.name;
    }
    //////////////////////////////////////////////////////
    // Implements Docker
    //////////////////////////////////////////////////////
    init(elm, parent) {
        super.init(elm, parent);
        this.name = elm.arg;
        this.townList.sort((t1, t2) => {
            let l1 = t1.getName().length;
            let l2 = t2.getName().length;
            return l1 > l2 ? -1 : (l1 == l2 ? 0 : -1);
        });
        for (const t of this.townList) {
            bayLog_1.BayLog.debug(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_SETTING_UP_TOWN, t.getName(), t.getLocation()));
        }
        this.defaultTown = bayDockers_1.BayDockers.createDockerByAlias("town", null);
        this.defaultClub = bayDockers_1.BayDockers.createDockerByAlias("club", "file");
    }
    //////////////////////////////////////////////////////
    // Implements DockerBase
    //////////////////////////////////////////////////////
    initDocker(dkr) {
        switch (dkr.getType()) {
            case "town":
                this.townList.push(dkr);
                break;
            case "club":
                this.clubList.push(dkr);
                break;
            case "log":
                this.logList.push(dkr);
                break;
            case "permission":
                this.permissionList.push(dkr);
                break;
            case "trouble":
                this.trouble = dkr;
                break;
            default:
                return false;
        }
        return true;
    }
    //////////////////////////////////////////////////////
    // Implements City
    //////////////////////////////////////////////////////
    getName() {
        return this.name;
    }
    getClubs() {
        return this.clubList;
    }
    getTowns() {
        return this.townList;
    }
    enter(tur) {
        bayLog_1.BayLog.debug("%s City[%s] Request URI: %s", tur, this.name, tur.req.uri);
        tur.city = this;
        for (const p of this.permissionList) {
            p.tourAdmitted(tur);
        }
        let mInfo = this.getTownAndClub(tur.req.uri);
        if (mInfo == null) {
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.NOT_FOUND, tur.req.uri);
        }
        mInfo.town.checkAdmitted(tur);
        if (mInfo.redirectURI != null) {
            throw httpException_1.HttpException.movedTemp(mInfo.redirectURI);
        }
        else {
            if (bayLog_1.BayLog.isDebug())
                bayLog_1.BayLog.debug("%s Town[%s] Club[%s]", tur, mInfo.town.getName(), mInfo.clubMatch.club);
            tur.req.queryString = mInfo.queryString;
            tur.req.scriptName = mInfo.clubMatch.scriptName;
            if (mInfo.clubMatch.club.getCharset() != null) {
                tur.req.charset = mInfo.clubMatch.club.getCharset();
                tur.res.charset = mInfo.clubMatch.club.getCharset();
            }
            else {
                tur.req.charset = bayserver_1.BayServer.harbor.getCharset();
                tur.res.charset = bayserver_1.BayServer.harbor.getCharset();
            }
            tur.req.pathInfo = mInfo.clubMatch.pathInfo;
            if (tur.req.pathInfo != null && mInfo.clubMatch.club.getrDecodePathInfo()) {
                try {
                    tur.req.pathInfo = decodeURI(tur.req.pathInfo);
                }
                catch (e) {
                    bayLog_1.BayLog.error_e(e);
                    try {
                        tur.req.pathInfo = decodeURI(tur.req.pathInfo);
                    }
                    catch (ee) {
                        bayLog_1.BayLog.error_e(ee);
                    }
                }
            }
            if (mInfo.rewrittenURI != null) {
                tur.req.rewrittenURI = mInfo.rewrittenURI; // URI is rewritten
            }
            let club = mInfo.clubMatch.club;
            tur.town = mInfo.town;
            tur.club = club;
            club.arrive(tur);
        }
    }
    getTrouble() {
        return this.trouble;
    }
    log(tour) {
        for (const d of this.logList) {
            try {
                d.log(tour);
            }
            catch (e) {
                bayLog_1.BayLog.error_e(e);
            }
        }
    }
    getType() {
        return "city";
    }
    //////////////////////////////////////////////////////
    // private methods
    //////////////////////////////////////////////////////
    clubMaches(clubList, relUri, townName) {
        let mi = new ClubMatchInfo();
        let anyd = null;
        for (const d of clubList) {
            if (d.getFileName() == "*" && d.getExtension() == null) {
                // Ignore any match club
                anyd = d;
                break;
            }
        }
        // search for club
        let parts = relUri.split("/");
        var relScriptName = "";
        loop: for (const fname of parts) {
            if (relScriptName != "")
                relScriptName += '/';
            relScriptName += fname;
            for (const d of clubList) {
                if (d === anyd) {
                    // Ignore any match club
                    continue;
                }
                if (d.matches(fname)) {
                    mi.club = d;
                    break loop;
                }
            }
        }
        if (mi.club == null && anyd != null) {
            mi.club = anyd;
        }
        if (mi.club == null)
            return null;
        if (townName == "/" && relScriptName == "") {
            mi.scriptName = "/";
            mi.pathInfo = null;
        }
        else {
            mi.scriptName = townName + relScriptName;
            if (relScriptName.length == relUri.length)
                mi.pathInfo = null;
            else
                mi.pathInfo = relUri.substring(relScriptName.length);
        }
        return mi;
    }
    /**
     * Determine club from request URI
     */
    getTownAndClub(reqUri) {
        if (reqUri == null)
            throw new Error("nullPo");
        let mi = new MatchInfo();
        var uri = reqUri;
        let pos = uri.indexOf('?');
        if (pos != -1) {
            mi.queryString = uri.substring(pos + 1);
            uri = uri.substring(0, pos);
        }
        for (const t of this.townList) {
            let m = t.matches(uri);
            if (m == town_1.MATCH_TYPE_NOT_MATCHED)
                continue;
            // town matched
            mi.town = t;
            if (m == town_1.MATCH_TYPE_CLOSE) {
                mi.redirectURI = uri + "/";
                if (mi.queryString != null)
                    mi.redirectURI += mi.queryString;
                return mi;
            }
            let orgUri = uri;
            uri = t.reroute(uri);
            if (uri != orgUri)
                mi.rewrittenURI = uri;
            let rel = uri.substring(t.getName().length);
            mi.clubMatch = this.clubMaches(t.getClubs(), rel, t.getName());
            if (mi.clubMatch == null) {
                mi.clubMatch = this.clubMaches(this.clubList, rel, t.getName());
            }
            if (mi.clubMatch == null) {
                // check index file
                if (uri.endsWith("/") && !strUtil_1.StrUtil.empty(t.getWelcomeFile())) {
                    let indexUri = uri + t.getWelcomeFile();
                    let relUri = rel + t.getWelcomeFile();
                    let indexLocation = sysUtil_1.SysUtil.joinPath(t.getLocation(), relUri);
                    if (sysUtil_1.SysUtil.isFile(indexLocation)) {
                        if (mi.queryString != null)
                            indexUri += "?" + mi.queryString;
                        let m2 = this.getTownAndClub(indexUri);
                        if (m2 != null) {
                            // matched
                            m2.rewrittenURI = indexUri;
                            return m2;
                        }
                    }
                }
                // default club matches
                mi.clubMatch = new ClubMatchInfo();
                mi.clubMatch.club = this.defaultClub;
                mi.clubMatch.scriptName = null;
                mi.clubMatch.pathInfo = null;
            }
            return mi;
        }
        return null;
    }
}
exports.BuiltInCityDocker = BuiltInCityDocker;
module.exports = {
    createDocker: () => new BuiltInCityDocker()
};
//# sourceMappingURL=builtInCityDocker.js.map