"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerNameItem = exports.RequestUrlItem = exports.RemoteUserItem = exports.IntervalItem = exports.TimeItem = exports.StatusItem = exports.StartLineItem = exports.QueryStringItem = exports.PortItem = exports.ResponseHeaderItem = exports.MethodItem = exports.RequestHeaderItem = exports.ProtocolItem = exports.RemoteLogItem = exports.FileNameItem = exports.ConnectionStatusItem = exports.RequestBytesItem2 = exports.RequestBytesItem1 = exports.ServerIpItem = exports.RemoteHostItem = exports.NullItem = exports.TextItem = void 0;
const logItem_1 = require("./logItem");
class TextItem extends logItem_1.LogItem {
    constructor(text) {
        super();
        this.text = text;
    }
    getItem(tour) {
        return this.text;
    }
}
exports.TextItem = TextItem;
/**
 * Return null result
 */
class NullItem extends logItem_1.LogItem {
    getItem(tour) {
        return null;
    }
}
exports.NullItem = NullItem;
NullItem.factory = () => new NullItem();
/**
 * Return remote IP address (%a)
 */
class RemoteHostItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.remoteAddress;
    }
}
exports.RemoteHostItem = RemoteHostItem;
RemoteHostItem.factory = () => new RemoteHostItem();
/**
 * Return local IP address (%A)
 */
class ServerIpItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.serverAddress;
    }
}
exports.ServerIpItem = ServerIpItem;
ServerIpItem.factory = () => new ServerIpItem();
/**
 * Return number of bytes that is sent from clients (Except HTTP headers)
 * (%B)
 */
class RequestBytesItem1 extends logItem_1.LogItem {
    getItem(tour) {
        let bytes = tour.req.headers.contentLength();
        if (bytes < 0)
            bytes = 0;
        return String(bytes);
    }
}
exports.RequestBytesItem1 = RequestBytesItem1;
RequestBytesItem1.factory = () => new RequestBytesItem1();
/**
 * Return number of bytes that is sent from clients in CLF format (Except
 * HTTP headers) (%b)
 */
class RequestBytesItem2 extends logItem_1.LogItem {
    getItem(tour) {
        let bytes = tour.req.headers.contentLength();
        if (bytes <= 0)
            return "-";
        else
            return String(bytes);
    }
}
exports.RequestBytesItem2 = RequestBytesItem2;
RequestBytesItem2.factory = () => new RequestBytesItem2();
/**
 * Return connection status (%c)
 */
class ConnectionStatusItem extends logItem_1.LogItem {
    getItem(tour) {
        if (tour.isAborted())
            return "X";
        else
            return "-";
    }
}
exports.ConnectionStatusItem = ConnectionStatusItem;
ConnectionStatusItem.factory = () => new ConnectionStatusItem();
/**
 * Return file name (%f)
 */
class FileNameItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.scriptName;
    }
}
exports.FileNameItem = FileNameItem;
FileNameItem.factory = () => new FileNameItem();
/**
 * Return remote log name (%l)
 */
class RemoteLogItem extends logItem_1.LogItem {
    getItem(tour) {
        return null;
    }
}
exports.RemoteLogItem = RemoteLogItem;
RemoteLogItem.factory = () => new RemoteLogItem();
/**
 * Return request protocol (%m)
 */
class ProtocolItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.protocol;
    }
}
exports.ProtocolItem = ProtocolItem;
ProtocolItem.factory = () => new ProtocolItem();
/**
 * Return requested header (%{Foobar}i)
 */
class RequestHeaderItem extends logItem_1.LogItem {
    init(param) {
        if (param == null)
            param = "";
        this.name = param;
    }
    getItem(tour) {
        return tour.req.headers.get(this.name);
    }
}
exports.RequestHeaderItem = RequestHeaderItem;
RequestHeaderItem.factory = () => new RequestHeaderItem();
/**
 * Return request method (%m)
 */
class MethodItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.method;
    }
}
exports.MethodItem = MethodItem;
MethodItem.factory = () => new MethodItem();
/**
 * Return responde header (%{Foobar}o)
 */
class ResponseHeaderItem extends logItem_1.LogItem {
    init(param) {
        if (param == null)
            param = "";
        this.name = param;
    }
    getItem(tour) {
        return tour.res.headers.get(this.name);
    }
}
exports.ResponseHeaderItem = ResponseHeaderItem;
ResponseHeaderItem.factory = () => new ResponseHeaderItem();
/**
 * The server port (%p)
 */
class PortItem extends logItem_1.LogItem {
    getItem(tour) {
        return String(tour.req.serverPort);
    }
}
exports.PortItem = PortItem;
PortItem.factory = () => new PortItem();
/**
 * Return query string (%q)
 */
class QueryStringItem extends logItem_1.LogItem {
    getItem(tour) {
        let qStr = tour.req.queryString;
        if (qStr != null)
            return '?' + qStr;
        else
            return null;
    }
}
exports.QueryStringItem = QueryStringItem;
QueryStringItem.factory = () => new QueryStringItem();
/**
 * The start line (%r)
 */
class StartLineItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.method + " " + tour.req.uri + " " + tour.req.protocol;
    }
}
exports.StartLineItem = StartLineItem;
StartLineItem.factory = () => new StartLineItem();
/**
 * Return status (%s)
 */
class StatusItem extends logItem_1.LogItem {
    getItem(tour) {
        return String(tour.res.headers.status);
    }
}
exports.StatusItem = StatusItem;
StatusItem.factory = () => new StatusItem();
/**
 * Return current time (%{format}t)
 */
class TimeItem extends logItem_1.LogItem {
    init(param) {
    }
    getItem(tour) {
        return "[" + new Date().toLocaleString() + "]";
    }
}
exports.TimeItem = TimeItem;
TimeItem.factory = () => new TimeItem();
/**
 * Return how long request took (%T)
 */
class IntervalItem extends logItem_1.LogItem {
    getItem(tour) {
        return String(tour.interval / 1000);
    }
}
exports.IntervalItem = IntervalItem;
IntervalItem.factory = () => new IntervalItem();
/**
 * Return remote user (%u)
 */
class RemoteUserItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.remoteUser;
    }
}
exports.RemoteUserItem = RemoteUserItem;
RemoteUserItem.factory = () => new RemoteUserItem();
/**
 * Return requested URL(not content query string) (%U)
 */
class RequestUrlItem extends logItem_1.LogItem {
    getItem(tour) {
        let url = tour.req.uri == null ? "" : tour.req.uri;
        let pos = url.indexOf('?');
        if (pos != -1)
            url = url.substring(0, pos);
        return url;
    }
}
exports.RequestUrlItem = RequestUrlItem;
RequestUrlItem.factory = () => new RequestUrlItem();
/**
 * Return the server name (%v)
 */
class ServerNameItem extends logItem_1.LogItem {
    getItem(tour) {
        return tour.req.serverName;
    }
}
exports.ServerNameItem = ServerNameItem;
ServerNameItem.factory = () => new ServerNameItem();
//# sourceMappingURL=logItems.js.map