"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarpData = void 0;
const httpHeaders_1 = require("../../util/httpHeaders");
const bayLog_1 = require("../../bayLog");
const sink_1 = require("../../sink");
class WarpData {
    constructor(warpShip, warpId) {
        this.reqHeaders = new httpHeaders_1.HttpHeaders();
        this.resHeaders = new httpHeaders_1.HttpHeaders();
        this.warpShip = warpShip;
        this.warpShipId = warpShip.id();
        this.warpId = warpId;
    }
    toString() {
        return this.warpShip + " wtur#" + this.warpId;
    }
    //////////////////////////////////////////////////////
    // Implements ReqContentHandler
    //////////////////////////////////////////////////////
    onReadContent(tur, buf, start, len) {
        bayLog_1.BayLog.debug("%s onReadReqContent tur=%s len=%d", this.warpShip, tur, len);
        this.warpShip.checkShipId(this.warpShipId);
        let maxLen = this.warpShip.protocolHandler.maxReqPacketDataSize();
        for (let pos = 0; pos < len; pos += maxLen) {
            let postLen = len - pos;
            if (postLen > maxLen) {
                postLen = maxLen;
            }
            let turId = tur.id();
            if (!this.started)
                // The buffer will become corrupted due to reuse.
                buf = Buffer.from(buf);
            this.warpShip.getWarpHandler().postWarpContents(tur, buf, start + pos, postLen, () => tur.req.consumed(turId, len));
        }
    }
    onEndContent(tur) {
        bayLog_1.BayLog.debug("%s endReqContent tur=%s", this.warpShip, tur);
        this.warpShip.checkShipId(this.warpShipId);
        this.warpShip.getWarpHandler().postWarpEnd(tur);
    }
    onAbort(tur) {
        bayLog_1.BayLog.debug("%s onAbortReq tur=%s", this.warpShip, tur);
        this.warpShip.checkShipId(this.warpShipId);
        bayLog_1.BayLog.debug("%s WarpShip abort", this.warpShip);
        this.warpShip.abort(this.warpShipId);
        return false; // not aborted immediately
    }
    //////////////////////////////////////////////////////
    // Other methods
    //////////////////////////////////////////////////////
    start() {
        if (this.started)
            throw new sink_1.Sink("%s Warp tour already started", this);
        bayLog_1.BayLog.debug("%s Start Warp tour", this);
        this.warpShip.flush();
        this.started = true;
    }
    static get(tur) {
        return tur.req.contentHandler;
    }
}
exports.WarpData = WarpData;
//# sourceMappingURL=warpData.js.map