"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarpDocker = void 0;
const clubBase_1 = require("../base/clubBase");
const grandAgent_1 = require("../../agent/grandAgent");
const net = require("net");
const strUtil_1 = require("../../util/strUtil");
const bayLog_1 = require("../../bayLog");
const warpShipStore_1 = require("./warpShipStore");
const protocolHandlerStore_1 = require("../../protocol/protocolHandlerStore");
const httpException_1 = require("../../httpException");
const httpStatus_1 = require("../../util/httpStatus");
const warpDataListener_1 = require("./warpDataListener");
const channelWrapper_1 = require("../../agent/channelWrapper");
class AgentListener {
    constructor(docker) {
        this.docker = docker;
    }
    add(agtId) {
        this.docker.stores.set(agtId, new warpShipStore_1.WarpShipStore(this.docker.maxShips));
    }
    remove(agtId) {
        this.docker.stores.delete(agtId);
    }
}
class WarpDocker extends clubBase_1.ClubBase {
    constructor() {
        super();
        this.port = -1;
        this.maxShips = -1;
        this.timeoutSec = -1; // -1 means "Use harbor.socketTimeoutSec"
        this.tourList = [];
        /** Agent ID => WarpShipStore */
        this.stores = null;
        this.stores = new Map();
    }
    //////////////////////////////////////////////////////
    // Implements Docker
    //////////////////////////////////////////////////////
    init(elm, parent) {
        super.init(elm, parent);
        if (strUtil_1.StrUtil.empty(this.warpBase))
            this.warpBase = "/";
        if (strUtil_1.StrUtil.isSet(this.host) && this.host.startsWith(":unix:")) {
            this.hostAddr = this.host.substring(6);
            this.port = -1;
        }
        else {
            if (this.port <= 0)
                this.port = 80;
            this.hostAddr = this.host;
        }
        grandAgent_1.GrandAgent.addLifecycleListener(new AgentListener(this));
    }
    //////////////////////////////////////////////////////
    // Implements DockerBase
    //////////////////////////////////////////////////////
    initKeyVal(kv) {
        switch (kv.key.toLowerCase()) {
            default:
                return super.initKeyVal(kv);
            case "destcity":
                this.host = kv.value;
                break;
            case "destport":
                this.port = Number.parseInt(kv.value);
                break;
            case "desttown":
                this.warpBase = kv.value;
                if (!this.warpBase.endsWith("/"))
                    this.warpBase += "/";
                break;
            case "maxships":
                this.maxShips = Number.parseInt(kv.value);
                break;
            case "timeout":
                this.timeoutSec = Number.parseInt(kv.value);
                break;
        }
        return true;
    }
    //////////////////////////////////////////////////////
    // Implements Club
    //////////////////////////////////////////////////////
    arrive(tour) {
        let agt = tour.ship.agent;
        let sto = this.getShipStore(agt.agentId);
        let wsip = sto.rent();
        if (wsip == null) {
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.SERVICE_UNAVAILABLE, "WarpDocker busy");
        }
        try {
            bayLog_1.BayLog.trace("%s got from store", wsip);
            let needConnect = false;
            let tp;
            if (!wsip.initialized) {
                let skt = new net.Socket();
                let ch = new channelWrapper_1.ChannelWrapper(skt);
                tp = this.newTransporter(agt, ch);
                let protoHnd = protocolHandlerStore_1.ProtocolHandlerStore.getStore(this.protocol(), false, agt.agentId).rent();
                wsip.initWarp(ch, agt, tp, this, protoHnd);
                tp.init(agt.nonBlockingHandler, ch, new warpDataListener_1.WarpDataListener(wsip));
                bayLog_1.BayLog.debug("%s init warp ship", wsip);
                needConnect = true;
            }
            this.tourList.push(tour);
            wsip.startWarpTour(tour);
            if (needConnect) {
                //agt.nonBlockingHandler.addChannelListener(wsip.ch, tp)
                agt.nonBlockingHandler.askToConnect(wsip.ch, this.hostAddr, this.port);
            }
        }
        catch (e) {
            bayLog_1.BayLog.error_e(e);
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.INTERNAL_SERVER_ERROR, e);
        }
    }
    //////////////////////////////////////////////////////
    // Other methods
    //////////////////////////////////////////////////////
    keepShip(wsip) {
        bayLog_1.BayLog.trace("%s keep warp ship: %s", this, wsip);
        this.getShipStore(wsip.agent.agentId).keep(wsip);
    }
    returnShip(wsip) {
        bayLog_1.BayLog.debug("%s return warp ship: %s", this, wsip);
        this.getShipStore(wsip.agent.agentId).Return(wsip);
    }
    returnProtocolHandler(agt, protoHnd) {
        bayLog_1.BayLog.debug("%s Return protocol handler: %s", agt, protoHnd);
        this.getProtocolHandlerStore(agt.agentId).Return(protoHnd);
    }
    getShipStore(agtId) {
        return this.stores.get(agtId);
    }
    //////////////////////////////////////////////////////
    // Private methods
    //////////////////////////////////////////////////////
    getProtocolHandlerStore(agtId) {
        return protocolHandlerStore_1.ProtocolHandlerStore.getStore(this.protocol(), false, agtId);
    }
}
exports.WarpDocker = WarpDocker;
//# sourceMappingURL=warpDocker.js.map