"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarpShipStore = void 0;
const objectStore_1 = require("../../util/objectStore");
const warpShip_1 = require("./warpShip");
const bayLog_1 = require("../../bayLog");
const arrayUtil_1 = require("../../util/arrayUtil");
const sink_1 = require("../../sink");
const strUtil_1 = require("../../util/strUtil");
class WarpShipStore extends objectStore_1.ObjectStore {
    constructor(maxShips) {
        super();
        this.keepList = [];
        this.busyList = [];
        this.maxShips = maxShips;
        this.factory = () => new warpShip_1.WarpShip();
    }
    rent() {
        if (this.maxShips > 0 && this.count() >= this.maxShips)
            return null;
        let wsip;
        if (arrayUtil_1.ArrayUtil.empty(this.keepList)) {
            bayLog_1.BayLog.trace("rent from Object Store");
            wsip = super.rent();
            if (wsip == null)
                return null;
        }
        else {
            bayLog_1.BayLog.trace("rent from freeList: %s", this.keepList);
            wsip = this.keepList.pop();
        }
        if (wsip == null)
            throw new sink_1.Sink("BUG! ship is null");
        if (wsip.postman != null && wsip.postman.isZombie())
            throw new sink_1.Sink("BUG! channel is zombie: " + wsip);
        this.busyList.push(wsip);
        bayLog_1.BayLog.trace("rent: keepList=%s busyList=%s", this.keepList, this.busyList);
        return wsip;
    }
    /**
     * Keep ship which connection is alive
     * @param wsip
     */
    keep(wsip) {
        bayLog_1.BayLog.trace("keep: before freeList=%s busyList=%s", this.keepList, this.busyList);
        if (!(arrayUtil_1.ArrayUtil.valueIn(wsip, this.busyList)))
            bayLog_1.BayLog.error("BUG: " + wsip + " not in busy list");
        arrayUtil_1.ArrayUtil.remove(wsip, this.busyList);
        this.keepList.push(wsip);
        bayLog_1.BayLog.trace("keep: after freeList=%s busyList=%s", this.keepList, this.busyList);
    }
    /**
     * Return ship which connection is closed
     * @param wsip
     */
    Return(wsip) {
        bayLog_1.BayLog.trace("Return: before keepList=%s busyList=%s", this.keepList, this.busyList);
        let removedFromFree = arrayUtil_1.ArrayUtil.remove(wsip, this.keepList);
        let removedFromBusy = arrayUtil_1.ArrayUtil.remove(wsip, this.busyList);
        if (!removedFromFree && !removedFromBusy)
            bayLog_1.BayLog.error("BUG:" + wsip + " not in both free list and busy list");
        super.Return(wsip);
        bayLog_1.BayLog.trace("Return: after keepList=%s busyList=%s", this.keepList, this.busyList);
    }
    count() {
        return this.keepList.length + this.busyList.length;
    }
    busyCount() {
        return this.busyList.length;
    }
    /**
     * print memory usage
     */
    printUsage(indent) {
        bayLog_1.BayLog.info("%sWarpShipStore Usage:", strUtil_1.StrUtil.indent(indent));
        bayLog_1.BayLog.info("%skeepList: %d", strUtil_1.StrUtil.indent(indent + 1), this.keepList.length);
        if (bayLog_1.BayLog.isDebug()) {
            for (const obj of this.keepList) {
                bayLog_1.BayLog.debug("%s%s", strUtil_1.StrUtil.indent(indent + 1), obj);
            }
        }
        bayLog_1.BayLog.info("%sbusyList: %d", strUtil_1.StrUtil.indent(indent + 1), this.busyList.length);
        if (bayLog_1.BayLog.isDebug()) {
            for (const obj of this.busyList) {
                bayLog_1.BayLog.debug("%s%s", strUtil_1.StrUtil.indent(indent + 1), obj);
            }
        }
        super.printUsage(indent);
    }
}
exports.WarpShipStore = WarpShipStore;
//# sourceMappingURL=warpShipStore.js.map