"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packet = void 0;
const buffer_1 = require("buffer");
const packetPartAccessor_1 = require("./packetPartAccessor");
/**
 * Packet format
 *   +---------------------------+
 *   +  Header(type, length etc) +
 *   +---------------------------+
 *   +  Data(payload data)       +
 *   +---------------------------+
 */
class Packet {
    constructor(type, headerLen, maxDataLen) {
        this.type = type;
        this.headerLen = headerLen;
        this.maxDataLen = maxDataLen;
        this.buf = buffer_1.Buffer.alloc(Packet.INITIAL_BUF_SIZE);
        this.bufLen = headerLen;
    }
    reset() {
        // Clear buffer for security
        this.buf.fill(0);
        this.bufLen = this.headerLen;
    }
    dataLen() {
        return this.bufLen - this.headerLen;
    }
    expand() {
        let newBuf = buffer_1.Buffer.alloc(this.buf.length * 2);
        this.buf.copy(newBuf, 0, 0);
        this.buf = newBuf;
    }
    newHeaderAccessor() {
        return new packetPartAccessor_1.PacketPartAccessor(this, 0, this.headerLen);
    }
    newDataAccessor() {
        return new packetPartAccessor_1.PacketPartAccessor(this, this.headerLen, -1);
    }
    toString() {
        return "Packet[" + this.type + "]";
    }
}
exports.Packet = Packet;
Packet.INITIAL_BUF_SIZE = 8192 * 4;
//# sourceMappingURL=packet.js.map