import { Packet } from "./packet";
import { Reusable } from "../util/Reusable";
import { PacketFactory } from "./packetFactory";
import { ObjectStore } from "../util/objectStore";
declare class ProtocolInfo<P extends Packet> {
    readonly protocol: string;
    readonly packetFactory: PacketFactory<P>;
    /** Agent ID => PacketStore */
    readonly stores: {
        [key: number]: PacketStore<P>;
    };
    constructor(proto: string, packetFactory: PacketFactory<P>);
    newStore(): PacketStore<P>;
    addAgent(agtId: number): void;
    removeAgent(agtId: number): void;
}
/**
 * Packet pool
 * @param <P> Packet
 * @param <T> Type of packet
 */
export declare class PacketStore<P extends Packet> implements Reusable {
    static protoMap: {
        [key: string]: ProtocolInfo<any>;
    };
    readonly protocol: string;
    readonly storeMap: {
        [key: number]: ObjectStore<P>;
    };
    readonly factory: PacketFactory<P>;
    constructor(protocol: string, factory: PacketFactory<P>);
    reset(): void;
    rent(typ: number): P;
    Return(pkt: P): void;
    /**
     * print memory usage
     */
    printUsage(indent: number): void;
    static init(): void;
    static registerProtocol(protocol: string, pktFactory: PacketFactory<any>): void;
    static getStore(protocol: string, agtId: number): PacketStore<any>;
    static getStores(agentId: number): PacketStore<any>[];
}
export {};
//# sourceMappingURL=packetStore.d.ts.map