"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProtocolHandlerStore = void 0;
const objectStore_1 = require("../util/objectStore");
const packetStore_1 = require("./packetStore");
const bayLog_1 = require("../bayLog");
const strUtil_1 = require("../util/strUtil");
const grandAgent_1 = require("../agent/grandAgent");
class AgentListener {
    add(agentId) {
        for (const info of ProtocolHandlerStore.protoMap.values()) {
            info.addAgent(agentId);
        }
    }
    remove(agentId) {
        for (const info of Object.values(ProtocolHandlerStore.protoMap)) {
            info.removeAgent(agentId);
        }
    }
}
class ProtocolInfo {
    constructor(proto, svrMode, protocolHandlerFactory) {
        /** Agent ID => ProtocolHandlerStore */
        this.stores = null;
        this.protocol = proto;
        this.serverMode = svrMode;
        this.protocolHandlerFactory = protocolHandlerFactory;
        this.stores = new Map();
    }
    addAgent(agtId) {
        let store = packetStore_1.PacketStore.getStore(this.protocol, agtId);
        this.stores.set(agtId, new ProtocolHandlerStore(this.protocol, this.serverMode, this.protocolHandlerFactory, store));
    }
    removeAgent(agtId) {
        this.stores.delete(agtId);
    }
}
class ProtocolHandlerStore extends objectStore_1.ObjectStore {
    constructor(protocol, svrMode, phFactory, pktStore) {
        super();
        this.protocol = protocol;
        this.serverMode = svrMode;
        this.factory = () => {
            return phFactory.createProtocolHandler(pktStore);
        };
    }
    /**
     * print memory usage
     */
    printUsage(indent) {
        bayLog_1.BayLog.info("%sProtocolHandlerStore(%s%s) Usage:", strUtil_1.StrUtil.indent(indent), this.protocol, this.serverMode ? "s" : "c");
        super.printUsage(indent + 1);
    }
    static init() {
        grandAgent_1.GrandAgent.addLifecycleListener(new AgentListener());
    }
    static getStore(proto, svrMode, agentId) {
        return this.protoMap.get(this.getProtocolForm(proto, svrMode)).stores.get(agentId);
    }
    static getStores(agentId) {
        let storeList = [];
        for (const info of this.protoMap.values()) {
            storeList.push(info.stores.get(agentId));
        }
        ;
        return storeList;
    }
    static registerProtocol(protocol, svrMode, pHndFactory) {
        let protoForm = this.getProtocolForm(protocol, svrMode);
        if (!(protoForm in this.protoMap)) {
            this.protoMap.set(protoForm, new ProtocolInfo(protocol, svrMode, pHndFactory));
        }
    }
    static getProtocolForm(protocol, svrMode) {
        if (svrMode)
            return protocol + "-s";
        else
            return protocol + "-c";
    }
    static initClass() {
        this.protoMap = new Map();
    }
}
exports.ProtocolHandlerStore = ProtocolHandlerStore;
ProtocolHandlerStore.protoMap = null;
ProtocolHandlerStore.initClass();
//# sourceMappingURL=protocolHandlerStore.js.map