"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TourRes = void 0;
const tour_1 = require("./tour");
const httpException_1 = require("../httpException");
const contentConsumeListener_1 = require("./contentConsumeListener");
const sysUtil_1 = require("../util/sysUtil");
const httpStatus_1 = require("../util/httpStatus");
const mimes_1 = require("../util/mimes");
const fs = require("fs");
const httpHeaders_1 = require("../util/httpHeaders");
const bayserver_1 = require("../bayserver");
const strUtil_1 = require("../util/strUtil");
const plainTransporter_1 = require("../agent/transporter/plainTransporter");
const sendFileYacht_1 = require("./sendFileYacht");
const bayLog_1 = require("../bayLog");
const sink_1 = require("../sink");
const protocolException_1 = require("../protocol/protocolException");
const channelWrapper_1 = require("../agent/channelWrapper");
const ioException_1 = require("../util/ioException");
class TourRes {
    //compressor] GzipCompressor;
    //yacht: SendFileYacht;
    constructor(tour) {
        /**
         * Response header info
         */
        this.headers = new httpHeaders_1.HttpHeaders();
        this.tour = tour;
    }
    init() {
        this.yacht = new sendFileYacht_1.SendFileYacht();
    }
    toString() {
        return this.tour.toString();
    }
    //////////////////////////////////////////////////////
    // Implements Reusable
    //////////////////////////////////////////////////////
    reset() {
        this.headers.clear();
        this.bytesPosted = 0;
        this.bytesConsumed = 0;
        this.bytesLimit = 0;
        this.charset = null;
        this.headerSent = false;
        this.yacht.reset();
        this.available = false;
        this.resConsumeListener = null;
        this.canCompress = false;
        //this.compressor = null;
    }
    sendHeaders(checkId) {
        this.tour.checkTourId(checkId);
        if (this.tour.isZombie())
            return;
        if (this.headerSent)
            return;
        bayLog_1.BayLog.trace("%s sendHeaders contLen=%d", this.tour, this.headers.contentLength());
        this.bytesLimit = this.headers.contentLength();
        // Compress check
        if (bayserver_1.BayServer.harbor.getGzipComp() &&
            this.headers.contains(httpHeaders_1.HttpHeaders.CONTENT_TYPE) &&
            this.headers.contentType().toLowerCase().startsWith("text/") &&
            !this.headers.contains(httpHeaders_1.HttpHeaders.CONTENT_ENCODING)) {
            let enc = this.tour.req.headers.get(httpHeaders_1.HttpHeaders.ACCEPT_ENCODING);
            if (enc != null) {
                let parts = enc.split(",");
                for (const part of parts) {
                    if (strUtil_1.StrUtil.eqIgnoreCase(part.trim(), "gzip")) {
                        this.canCompress = true;
                        this.headers.set(httpHeaders_1.HttpHeaders.CONTENT_ENCODING, "gzip");
                        this.headers.remove(httpHeaders_1.HttpHeaders.CONTENT_LENGTH);
                        break;
                    }
                }
            }
        }
        try {
            this.tour.ship.sendHeaders(this.tour.shipId, this.tour);
        }
        catch (e) {
            bayLog_1.BayLog.debug("%s Error on sending headers: %s", this.tour, e.message);
            this.tour.changeState(tour_1.Tour.TOUR_ID_NOCHECK, tour_1.Tour.STATE_ABORTED);
            throw e;
        }
        finally {
            this.headerSent = true;
        }
    }
    setConsumeListener(listener) {
        this.resConsumeListener = listener;
        this.bytesConsumed = 0;
        this.bytesPosted = 0;
        this.available = true;
    }
    sendContent(checkId, buf, ofs, len) {
        if (buf == null)
            throw new Error("nullPo");
        this.tour.checkTourId(checkId);
        bayLog_1.BayLog.trace("%s send content: len=%d", this, len);
        // New listener
        let lis = () => {
            //BayLog.debug("%s call back. consumed", this.tour)
            this.consumed(checkId, len);
        };
        if (this.tour.isZombie()) {
            bayLog_1.BayLog.debug("%s zombie return", this);
            lis();
            return true;
        }
        if (!this.headerSent)
            throw new sink_1.Sink("BUG!: Header not sent");
        if (this.resConsumeListener == null)
            throw new sink_1.Sink("Response consume listener is null");
        this.bytesPosted += len;
        bayLog_1.BayLog.debug("%s posted res content len=%d posted=%d limit=%d consumed=%d", this.tour, len, this.bytesPosted, this.bytesLimit, this.bytesConsumed);
        if (this.bytesLimit > 0 && this.bytesPosted > this.bytesLimit) {
            throw new protocolException_1.ProtocolException("Post data exceed content-length: " + this.bytesPosted + "/" + this.bytesLimit);
        }
        if (this.tour.isZombie() || this.tour.isAborted()) {
            // Don't send peer any data. Do nothing
            bayLog_1.BayLog.debug("%s Aborted or zombie tour. do nothing: %s state=%s", this, this.tour, this.tour.state);
            lis();
        }
        else {
            if (this.canCompress) {
                //this.getCompressor().compress(buf, ofs, len, lis);
            }
            else {
                try {
                    this.tour.ship.sendResContent(this.tour.shipId, this.tour, buf, ofs, len, lis);
                }
                catch (e) {
                    lis();
                    this.tour.changeState(tour_1.Tour.TOUR_ID_NOCHECK, tour_1.Tour.STATE_ABORTED);
                    throw e;
                }
            }
        }
        let oldAvailable = this.available;
        if (!this.bufferAvailable())
            this.available = false;
        if (oldAvailable && !this.available)
            bayLog_1.BayLog.debug("%s response unavailable (_ _): posted=%d consumed=%d", this, this.bytesPosted, this.bytesConsumed);
        return this.available;
    }
    endContent(chkId) {
        this.tour.checkTourId(chkId);
        bayLog_1.BayLog.debug("%s end ResContent", this);
        if (this.tour.isEnded()) {
            bayLog_1.BayLog.debug("%s Tour is already ended (Ignore).", this);
            return;
        }
        if (!this.tour.isZombie() && this.tour.city != null)
            this.tour.city.log(this.tour);
        // send end message
        if (this.canCompress) {
            //this.getCompressor().finish();
        }
        let tourReturned = false;
        let callback = () => {
            this.tour.checkTourId(chkId);
            this.tour.ship.returnTour(this.tour);
            tourReturned = true;
        };
        try {
            if (this.tour.isZombie() || this.tour.isAborted()) {
                // Don't send peer any data. Only return tour
                bayLog_1.BayLog.debug("%s Aborted or zombie tour. do nothing: %s state=%s", this, this.tour, this.tour.state);
                callback();
            }
            else {
                try {
                    this.tour.ship.sendEndTour(this.tour.shipId, this.tour, callback);
                }
                catch (e) {
                    if (!(e instanceof ioException_1.IOException))
                        throw e;
                    else {
                        bayLog_1.BayLog.debug("%s Error on sending end tour", this);
                        callback();
                        throw e;
                    }
                }
            }
        }
        finally {
            // If tour is returned, we cannot change its state because
            // it will become uninitialized.
            bayLog_1.BayLog.debug("%s Tour is returned (id=%d): %s (state=%d)", this, chkId, tourReturned, this.tour.state);
            if (!tourReturned) {
                this.tour.changeState(tour_1.Tour.TOUR_ID_NOCHECK, tour_1.Tour.STATE_ENDED);
            }
        }
    }
    //////////////////////////////////////////////////////
    // Methods to sending error
    //////////////////////////////////////////////////////
    sendHttpException(chkId, e) {
        if (e.status == httpStatus_1.HttpStatus.MOVED_TEMPORARILY || e.status == httpStatus_1.HttpStatus.MOVED_PERMANENTLY)
            this.sendRedirect(chkId, e.status, e.location);
        else
            this.sendError(chkId, e.status, e.message, e);
    }
    sendError(chkId, status = httpStatus_1.HttpStatus.INTERNAL_SERVER_ERROR, message = "", e = null) {
        this.tour.checkTourId(chkId);
        if (this.tour.isZombie())
            return;
        if (this.headerSent) {
            bayLog_1.BayLog.debug("Try to send error after response header is sent (Ignore)");
            bayLog_1.BayLog.debug("%s: status=%d, message=%s", this, status, message);
            if (e != null)
                bayLog_1.BayLog.debug_e(e);
        }
        else {
            this.setConsumeListener(contentConsumeListener_1.ContentConsumeListenerUtil.devNull);
            if (this.tour.isZombie() || this.tour.isAborted()) {
                // Don't send peer any data. Do nothing
                bayLog_1.BayLog.debug("%s Bborted or zombie tour. do nothing: %s state=%s", this, this.tour, this.tour.state);
            }
            else {
                try {
                    this.tour.ship.sendError(this.tour.shipId, this.tour, status, message, e);
                }
                catch (e) {
                    if (e instanceof ioException_1.IOException) {
                        bayLog_1.BayLog.debug_e(e, "%s IOException while sending error", this);
                        this.tour.changeState(tour_1.Tour.TOUR_ID_NOCHECK, tour_1.Tour.STATE_ABORTED);
                    }
                    else {
                        throw e;
                    }
                }
                this.headerSent = true;
            }
        }
        this.endContent(chkId);
    }
    //////////////////////////////////////////////////////
    // Sending file methods
    //////////////////////////////////////////////////////
    sendFile(chkId, file, charset, async) {
        this.tour.checkTourId(chkId);
        if (this.tour.isZombie())
            return;
        if (sysUtil_1.SysUtil.isDirectory(file)) {
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.FORBIDDEN, file);
        }
        else if (!sysUtil_1.SysUtil.exists(file)) {
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.NOT_FOUND, file);
        }
        var mimeType = null;
        let rname = file;
        let pos = rname.lastIndexOf('.');
        if (pos >= 0) {
            let ext = rname.substring(pos + 1).toLowerCase();
            mimeType = mimes_1.Mimes.getType(ext);
        }
        if (mimeType == null)
            mimeType = "application/octet-stream";
        if (mimeType.startsWith("text/") && strUtil_1.StrUtil.isSet(this.charset))
            mimeType = mimeType + "; charset=" + charset;
        //resHeaders.setStatus(HttpStatus.OK);
        let fileSize = sysUtil_1.SysUtil.getFileSize(file);
        this.headers.setContentType(mimeType);
        this.headers.setContentLength(fileSize);
        try {
            this.sendHeaders(tour_1.Tour.TOUR_ID_NOCHECK);
            let bufsize = this.tour.ship.protocolHandler.maxResPacketDataSize();
            let fd = fs.openSync(file, "r");
            let buf = Buffer.alloc(8192);
            let tp = new plainTransporter_1.PlainTransporter(false, bufsize);
            this.yacht.init(this.tour, file, fileSize, tp);
            let ch = new channelWrapper_1.ChannelWrapper(fd);
            tp.init(this.tour.ship.agent.nonBlockingHandler, ch, this.yacht);
            tp.openValve();
        }
        catch (e) {
            bayLog_1.BayLog.error_e(e);
            throw new httpException_1.HttpException(httpStatus_1.HttpStatus.INTERNAL_SERVER_ERROR, file);
        }
    }
    sendRedirect(chkId, status, location) {
        this.tour.checkTourId(chkId);
        if (this.headerSent) {
            bayLog_1.BayLog.error("Try to redirect after response header is sent (Ignore)");
        }
        else {
            this.setConsumeListener(contentConsumeListener_1.ContentConsumeListenerUtil.devNull);
            this.tour.ship.sendRedirect(this.tour.shipId, this.tour, status, location);
            this.headerSent = true;
        }
        this.endContent(chkId);
    }
    consumed(chkId, len) {
        this.tour.checkTourId(chkId);
        if (this.resConsumeListener == null)
            throw new sink_1.Sink("Response consume listener is null");
        this.bytesConsumed += len;
        bayLog_1.BayLog.debug("%s resConsumed: len=%d posted=%d consumed=%d limit=%d", this.tour, len, this.bytesPosted, this.bytesConsumed, this.bytesLimit);
        var resume = false;
        var oldAvailable = this.available;
        if (this.bufferAvailable())
            this.available = true;
        if (!oldAvailable && this.available) {
            bayLog_1.BayLog.debug("%s response available (^o^): posted=%d consumed=%d", this, this.bytesPosted, this.bytesConsumed);
            resume = true;
        }
        if (this.tour.isRunning()) {
            this.resConsumeListener(len, resume);
        }
    }
    bufferAvailable() {
        return this.bytesPosted - this.bytesConsumed < bayserver_1.BayServer.harbor.getTourBufferSize();
    }
}
exports.TourRes = TourRes;
//# sourceMappingURL=tourRes.js.map