"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpHeaders = void 0;
const httpStatus_1 = require("./httpStatus");
const strUtil_1 = require("./strUtil");
class HttpHeaders {
    constructor() {
        /** Status */
        this.status = httpStatus_1.HttpStatus.OK;
        /** Header hash */
        this.headers = null;
        this.headers = new Map();
    }
    toString() {
        return "Headers(s=" + this.status + " h=" + this.headers;
    }
    copyTo(dst) {
        dst.status = this.status;
        for (const [name, values] of this.headers) {
            let newValues = [...values];
            dst.headers.set(name, newValues);
        }
    }
    /**
     * Get the header value as string
     */
    get(name) {
        if (name == null)
            throw new Error("nullPo");
        let values = this.headers.get(strUtil_1.StrUtil.toLowerCase(name));
        if (values == null)
            return null;
        return values[0];
    }
    /**
     * Get the header value as int
     */
    getInt(name) {
        let val = this.get(name);
        if (val == null)
            return -1;
        else
            return Number.parseInt(val);
    }
    /**
     * Update the header value by string
     */
    set(name, value) {
        if (name == null)
            throw new Error("nullPo");
        if (value == null)
            throw new Error("nullPo");
        name = strUtil_1.StrUtil.toLowerCase(name);
        let values = this.headers.get(name);
        if (values == null) {
            values = [];
            this.headers.set(name, values);
        }
        values.length = 0;
        values.push(value);
    }
    /**
     * Add a header value by string
     */
    add(name, value) {
        if (name == null)
            throw new Error("nullPo");
        if (value == null)
            throw new Error("nullPo");
        name = strUtil_1.StrUtil.toLowerCase(name);
        let values = this.headers.get(name);
        if (values == null) {
            values = [];
            this.headers.set(name, values);
        }
        values.push(value);
    }
    /**
     * Get all the header name
     */
    names() {
        let names = [];
        for (const name of this.headers.keys()) {
            names.push(name);
        }
        return names;
    }
    /**
     * Get all the header values of specified header name
     */
    values(name) {
        let values = this.headers.get(strUtil_1.StrUtil.toLowerCase(name));
        if (values == null)
            return [];
        else
            return values;
    }
    /**
     * Check the existence of header
     */
    contains(name) {
        if (name == null)
            throw new Error("nullPo");
        return this.headers.has(strUtil_1.StrUtil.toLowerCase(name));
    }
    remove(name) {
        if (name == null)
            throw new Error("nullPo");
        this.headers.delete(name);
    }
    //////////////////////////////////////////////////////
    // Useful methods
    //////////////////////////////////////////////////////
    contentType() {
        return this.get(HttpHeaders.CONTENT_TYPE);
    }
    setContentType(type) {
        this.set(HttpHeaders.CONTENT_TYPE, type);
    }
    /**
     * Get content length
     *
     * @return content length. If there isn't a content length header, return
     *         -1.
     */
    contentLength() {
        let length = this.get(HttpHeaders.CONTENT_LENGTH);
        if (strUtil_1.StrUtil.empty(length))
            return -1;
        else
            return Number.parseInt(length);
    }
    setContentLength(length) {
        this.set(HttpHeaders.CONTENT_LENGTH, length.toString());
    }
    getConnection() {
        let con = this.get(HttpHeaders.CONNECTION);
        if (con !== null)
            con = strUtil_1.StrUtil.toLowerCase(con);
        switch (con) {
            case "close":
                return HttpHeaders.CONNECTION_CLOSE;
            case "keep-alive":
                return HttpHeaders.CONNECTION_KEEP_ALIVE;
            case "upgrade":
                return HttpHeaders.CONNECTION_UPGRADE;
            default:
                return HttpHeaders.CONNECTION_UNKOWN;
        }
    }
    clear() {
        this.headers.clear();
        this.status = httpStatus_1.HttpStatus.OK;
    }
}
exports.HttpHeaders = HttpHeaders;
/**
 * Known header names
 */
HttpHeaders.HEADER_SEPARATOR = ": ";
HttpHeaders.HEADER_SEPARATOR_BYTES = HttpHeaders.HEADER_SEPARATOR.charCodeAt(0);
HttpHeaders.CONTENT_TYPE = "content-type";
HttpHeaders.CONTENT_LENGTH = "content-length";
HttpHeaders.CONTENT_ENCODING = "content-encoding";
HttpHeaders.HDR_TRANSFER_ENCODING = "Transfer-Encoding";
HttpHeaders.CONNECTION = "Connection";
HttpHeaders.AUTHORIZATION = "Authorization";
HttpHeaders.WWW_AUTHENTICATE = "WWW-Authenticate";
HttpHeaders.STATUS = "Status";
HttpHeaders.LOCATION = "Location";
HttpHeaders.HOST = "Host";
HttpHeaders.COOKIE = "Cookie";
HttpHeaders.USER_AGENT = "User-Agent";
HttpHeaders.ACCEPT = "Accept";
HttpHeaders.ACCEPT_LANGUAGE = "Accept-Language";
HttpHeaders.ACCEPT_ENCODING = "Accept-Encoding";
HttpHeaders.UPGRADE_INSECURE_REQUESTS = "Upgrade-Insecure-Requests";
HttpHeaders.SERVER = "Server";
HttpHeaders.X_FORWARDED_HOST = "X-Forwarded-Host";
HttpHeaders.X_FORWARDED_FOR = "X-Forwarded-For";
HttpHeaders.X_FORWARDED_PROTO = "X-Forwarded-Proto";
HttpHeaders.X_FORWARDED_PORT = "X-Forwarded-Port";
HttpHeaders.CONNECTION_CLOSE = 1;
HttpHeaders.CONNECTION_KEEP_ALIVE = 2;
HttpHeaders.CONNECTION_UPGRADE = 3;
HttpHeaders.CONNECTION_UNKOWN = 4;
//# sourceMappingURL=httpHeaders.js.map