"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpUtil = void 0;
const httpHeaders_1 = require("./httpHeaders");
const strUtil_1 = require("./strUtil");
const bayLog_1 = require("../bayLog");
const charUtil_1 = require("./charUtil");
class HttpUtil {
    /**
     * Send MIME headers This method is called from sendHeaders()
     */
    static sendMimeHeaders(hdr, out) {
        // headers
        for (const name of hdr.names()) {
            for (const value of hdr.values(name)) {
                out.put(strUtil_1.StrUtil.toBytes(name));
                out.putByte(httpHeaders_1.HttpHeaders.HEADER_SEPARATOR_BYTES);
                out.put(strUtil_1.StrUtil.toBytes(value));
                HttpUtil.sendNewLine(out);
            }
        }
    }
    static sendNewLine(buf) {
        buf.put(charUtil_1.CharUtil.CRLF_BYTES);
    }
    /**
     * Parse AUTHORIZATION header
     */
    static parseAuthrization(tur) {
        let auth = tur.req.headers.get(httpHeaders_1.HttpHeaders.AUTHORIZATION);
        if (!strUtil_1.StrUtil.empty(auth)) {
            let mch = auth.match(/Basic (.*)/);
            if (!mch) {
                bayLog_1.BayLog.warn("Not matched with basic authentication format");
            }
            else {
                auth = mch[1];
                try {
                    auth = atob(auth);
                    mch = auth.match(/(.*):(.*)/);
                    if (mch) {
                        tur.req.remoteUser = mch[1];
                        tur.req.remotePass = mch[2];
                    }
                }
                catch (e) {
                    bayLog_1.BayLog.error_e(e);
                }
            }
        }
    }
    static parseHostPort(tour, defaultPort) {
        tour.req.reqHost = "";
        let hostPort = tour.req.headers.get(httpHeaders_1.HttpHeaders.X_FORWARDED_HOST);
        if (strUtil_1.StrUtil.isSet(hostPort)) {
            tour.req.headers.remove(httpHeaders_1.HttpHeaders.X_FORWARDED_HOST);
            tour.req.headers.set(httpHeaders_1.HttpHeaders.HOST, hostPort);
        }
        hostPort = tour.req.headers.get(httpHeaders_1.HttpHeaders.HOST);
        if (strUtil_1.StrUtil.isSet(hostPort)) {
            let pos = hostPort.lastIndexOf(':');
            if (pos == -1) {
                tour.req.reqHost = hostPort;
                tour.req.reqPort = defaultPort;
            }
            else {
                tour.req.reqHost = hostPort.substring(0, pos);
                try {
                    tour.req.reqPort = Number.parseInt(hostPort.substring(pos + 1));
                }
                catch (e) {
                    bayLog_1.BayLog.error_e(e);
                }
            }
        }
    }
    static resolveHost(adr) {
        return adr;
    }
}
exports.HttpUtil = HttpUtil;
HttpUtil.MAX_LINE_LEN = 5000;
//# sourceMappingURL=httpUtil.js.map