"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IOUtil = void 0;
const buffer_1 = require("buffer");
const charUtil_1 = require("./charUtil");
const net = require("net");
const bayLog_1 = require("../bayLog");
const bayMessage_1 = require("../bayMessage");
const symbol_1 = require("../symbol");
const net_1 = require("net");
const bayException_1 = require("../bayException");
const ioException_1 = require("./ioException");
const process_1 = require("process");
class IOUtil {
    static readLine(r) {
        var line = [];
        while (true) {
            let c = r.readInt8();
            if (c == null)
                break;
            else if (c == charUtil_1.CharUtil.LF_CODE)
                break;
            else if (c == charUtil_1.CharUtil.CR_CODE)
                continue;
            line.push(c);
        }
        return String.fromCodePoint(...line);
    }
    static getSockRecvBufSize(skt) {
        return skt.writableLength;
    }
    static readInt32(buf) {
        if (buf.length != 4)
            throw new ioException_1.IOException("Invalid buffer length: " + buf.length);
        return buf.readInt32BE(0);
    }
    static writeInt32(writeCh, data) {
        let buf = buffer_1.Buffer.alloc(4);
        buf.writeInt32BE(data);
        writeCh.write(buf);
    }
    static readInt64(buf, pos) {
        if (buf.length - pos < 8)
            throw new ioException_1.IOException("Invalid buffer length: " + buf.length);
        return buf.readBigUInt64BE(pos);
    }
    static writeInt64(writeCh, data) {
        let buf = buffer_1.Buffer.alloc(8);
        buf.writeBigUInt64BE(data);
        writeCh.write(buf);
    }
    static openLocalPipe(serverSideSocketCatcher, clientSideSocketCatcher) {
        // Dynamic and/or Private Ports (49152-65535)
        // https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xhtml
        let DYNAMIC_PORTS_START = 49152;
        for (let i = DYNAMIC_PORTS_START; i < 65535 + 1; i++) {
            try {
                this.openLocalPipeByPort(i, serverSideSocketCatcher, clientSideSocketCatcher);
                return;
            }
            catch (e) {
                bayLog_1.BayLog.error_e(e);
                continue;
            }
        }
        throw new bayException_1.BayException("Cannot create local pipe");
    }
    static openLocalPipeByPort(portNum, serverSideSocketCatcher, clientSideSocketCatcher) {
        let localhost = "127.0.0.1";
        let serverSkt = (0, net_1.createServer)((skt) => {
            serverSideSocketCatcher(skt);
            if (true) {
                bayLog_1.BayLog.debug(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_CLOSING_LOCAL_PORT, portNum));
                serverSkt.close();
            }
        });
        serverSkt.on('error', (err) => {
            //BayLog.error_e(err, "Server side")
            if (portNum == 65535) {
                this.openFail();
            }
            this.openLocalPipeByPort(portNum + 1, serverSideSocketCatcher, clientSideSocketCatcher);
        });
        serverSkt.listen(portNum, "127.0.0.1", () => {
            bayLog_1.BayLog.debug(bayMessage_1.BayMessage.get(symbol_1.Symbol.MSG_OPENING_LOCAL_PORT, portNum));
            // Connect from client
            let clientSideSkt = net.connect(portNum, "127.0.0.1", () => {
                clientSideSocketCatcher(clientSideSkt);
            });
            clientSideSkt.on('error', (err) => {
                if (portNum == 65535)
                    this.openFail();
            });
        });
    }
    static openFail() {
        bayLog_1.BayLog.error("Failed to open local port");
        (0, process_1.exit)(1);
    }
}
exports.IOUtil = IOUtil;
//# sourceMappingURL=ioUtil.js.map